@@ -314,7 +314,7 @@ def _process_file_or_entry_point(parser):
             env = env._replace_flatten_keys(replaced)
             dag = DAGSpec(dag_dict, env=env).to_dag()
         else:
-            dag = DAGSpec(dag_dict, env=None).to_dag()
+            dag = DAGSpec(dag_dict).to_dag()
 
     return dag, args
 
