@@ -10,7 +10,6 @@ class ProductsContainer:
     keys), this non-standard behavior but it is needed to simpplify the
     MetaProduct API since it has to work with lists and dictionaries
     """
-
     def __init__(self, products):
         self.products = products
 
@@ -29,8 +28,10 @@ class ProductsContainer:
         """Returns a JSON serializable version of this product
         """
         if isinstance(self.products, Mapping):
-            return {name: str(product) for name, product
-                    in self.products.items()}
+            return {
+                name: str(product)
+                for name, product in self.products.items()
+            }
         else:
             return list(str(product) for product in self.products)
 
@@ -38,14 +39,14 @@ class ProductsContainer:
         return len(self.products)
 
     def __repr__(self):
-        return '{}: {}'.format(type(self).__name__, repr(self.products))
+        return '{}({})'.format(type(self).__name__, str(self.products))
 
     def __str__(self):
         return str(self.products)
 
 
 # NOTE: rename this to ProductCollection?
-class MetaProduct:
+class MetaProduct(Mapping):
     """
     Exposes a Product-like API to allow Tasks to create more than one Product,
     it is automatically instantiated when a Task is initialized with a
@@ -55,21 +56,12 @@ class MetaProduct:
     (executed via NotebookRunner), for fitting a model might as well serialize
     the things such as the model and any data preprocessors
     """
-
     def __init__(self, products):
         container = ProductsContainer(products)
 
         self.products = container
         self.metadata = MetadataCollection(container)
 
-    @property
-    def timestamp(self):
-        return self.metadata.timestamp
-
-    @property
-    def stored_source_code(self):
-        return self.metadata.stored_source_code
-
     @property
     def task(self):
         # TODO: validate same task
@@ -80,16 +72,6 @@ class MetaProduct:
         for p in self.products:
             p.task = value
 
-    @timestamp.setter
-    def timestamp(self, value):
-        for p in self.products:
-            p.metadata['timestamp'] = value
-
-    @stored_source_code.setter
-    def stored_source_code(self, value):
-        for p in self.products:
-            p.metadata['stored_source_code'] = value
-
     def exists(self):
         return all([p.exists() for p in self.products])
 
@@ -98,20 +80,16 @@ class MetaProduct:
             product.delete(force)
 
     def _is_outdated(self, outdated_by_code=True):
-        return any([p._is_outdated(outdated_by_code=outdated_by_code)
-                    for p in self.products])
+        return any([
+            p._is_outdated(outdated_by_code=outdated_by_code)
+            for p in self.products
+        ])
 
     def _outdated_data_dependencies(self):
-        return any([p._outdated_data_dependencies()
-                    for p in self.products])
+        return any([p._outdated_data_dependencies() for p in self.products])
 
     def _outdated_code_dependency(self):
-        return any([p._outdated_code_dependency()
-                    for p in self.products])
-
-    def _clear_cached_status(self):
-        for p in self.products:
-            p._clear_cached_status()
+        return any([p._outdated_code_dependency() for p in self.products])
 
     def to_json_serializable(self):
         """Returns a JSON serializable version of this product
@@ -120,15 +98,12 @@ class MetaProduct:
         # are supported such as NotebookRunner that depends on papermill
         return self.products.to_json_serializable()
 
-    def _save_metadata(self, source_code):
-        self.metadata.update(source_code)
-
     def render(self, params, **kwargs):
         for p in self.products:
             p.render(params, **kwargs)
 
     def __repr__(self):
-        return '{}: {}'.format(type(self).__name__, str(self.products))
+        return '{}({})'.format(type(self).__name__, str(self.products))
 
     def __str__(self):
         return str(self.products)
@@ -140,11 +115,5 @@ class MetaProduct:
     def __getitem__(self, key):
         return self.products[key]
 
-    def get(self, key):
-        try:
-            return self[key]
-        except KeyError:
-            return None
-
     def __len__(self):
         return len(self.products)
