@@ -28,7 +28,6 @@ class Product(abc.ABC):
         any of the existing keys but additional key-value pairs might be
         included
     """
-
     def __init__(self, identifier):
         self._identifier = self._init_identifier(identifier)
 
@@ -46,10 +45,6 @@ class Product(abc.ABC):
 
         self.prepare_metadata = _prepare_metadata
 
-    def _save_metadata(self, source_code):
-        # this is called by task in the exec_status setter
-        self.metadata.update(source_code)
-
     @property
     def task(self):
         if self._task is None:
@@ -79,38 +74,32 @@ class Product(abc.ABC):
         bool
             True if the Task should execute
         """
-        run = False
-
-        self.logger.info('Checking status for task "%s"', self.task.name)
-
         # check product...
         p_exists = self.exists()
 
         # check dependencies only if the product exists
         if p_exists:
 
-            if self._outdated_data_dependencies():
-                run = True
-                self.logger.info('Outdated data deps...')
+            oudated_data = self._outdated_data_dependencies()
+            outdated_code = (outdated_by_code
+                             and self._outdated_code_dependency())
+            run = oudated_data or outdated_code
+
+            if run:
+                self.logger.info(
+                    'Task "%s" is outdated, it will be executed...',
+                    self.task.name)
             else:
-                self.logger.info('Up-to-date data deps...')
+                self.logger.info(
+                    'Task "%s" is up-to-date, it will be skipped...',
+                    self.task.name)
 
-            if outdated_by_code:
-                if self._outdated_code_dependency():
-                    run = True
-                    self.logger.info('Outdated code dep...')
-                else:
-                    self.logger.info('Up-to-date code dep...')
+            return run
         else:
-            run = True
-
-            # just log why it will run
-            if not p_exists:
-                self.logger.info('Product does not exist...')
-
-        self.logger.info('Should run? %s', run)
-
-        return run
+            self.logger.info(
+                'Product of task "%s" does not exist, it will be executed...',
+                self.task.name)
+            return True
 
     def _outdated_data_dependencies(self):
         """
@@ -135,8 +124,8 @@ class Product(abc.ABC):
                 return ((up_prod.metadata.timestamp > self.metadata.timestamp)
                         or up_prod._is_outdated())
 
-        outdated = any([is_outdated(up.product) for up
-                        in self.task.upstream.values()])
+        outdated = any(
+            [is_outdated(up.product) for up in self.task.upstream.values()])
 
         self._outdated_data_dependencies_status = outdated
 
@@ -165,8 +154,7 @@ class Product(abc.ABC):
         self._outdated_code_dependency_status = outdated
 
         self.logger.debug(('Finished checking code status for task "%s" '
-                           'Outdated? %s'),
-                          self.task.name,
+                           'Outdated? %s'), self.task.name,
                           self._outdated_code_dependency_status)
 
         if outdated:
@@ -175,22 +163,12 @@ class Product(abc.ABC):
 
         return self._outdated_code_dependency_status
 
-    def _clear_cached_status(self):
-        # These flags keep a cache of the Product's outdated status, they
-        # are computed using the Product's metadata, hence they will only
-        # change when the metadata changes. Metadata changes in three
-        # situations: 1) at startup (loaded from disk), 2) after build
-        # (metadata is updated and then saved to disk) and 3) Forced load
-        # (if we force loading, currently not implemented).
-        self._outdated_data_dependencies_status = None
-        self._outdated_code_dependency_status = None
-
     def __str__(self):
         return str(self._identifier)
 
     def __repr__(self):
-        return '{}({})'.format(type(self).__name__,
-                               self._identifier.best_str(shorten=True))
+        return '{}({})'.format(
+            type(self).__name__, self._identifier.best_str(shorten=True))
 
     def __getstate__(self):
         state = self.__dict__.copy()
@@ -243,3 +221,11 @@ class Product(abc.ABC):
         """Deletes the product
         """
         pass
+
+    # NOTE: currently optional but there is a conflict with this. metadata
+    # defines a delete public method which calls product._delete_metadata
+    # but not all products implement this
+    def _delete_metadata(self):
+        raise NotImplementedError(
+            '_delete_metadata not implemented in {}'.format(
+                type(self).__name__))
