@@ -1,11 +1,60 @@
 """
-Notes to developers
+Path management
+---------------
+Relative paths are heavily used for pointing to source files and products,
+when only the Python API existed, this was unambiguous: all paths
+were relative to the current working directory, you only had to make sure
+to start the session at the right place. Initially, the Spec API didn't
+change a thing since the assumption was that one would convert a spec to
+a DAG in a folder containing a pipeline.yaml, under this scenario the
+current directory is also the parent of pipeline.yaml so all relative paths
+work as expected.
 
-meta:
-  Settings that cannot be clearly mapped to the OOP Python interface, we
-  don't call it config because there is a DAGConfig object in the Python
-  API and this might cause confusion
+However, when we introduced the Jupyter custom contents manager things
+changed because the DAG can be initialized from directories where there
+isn't a pipeline.yaml but we have to recursively look for one. Furthermore,
+if extract_upstream or extract_product are True and the source files do
+not share the parent with pipeline.yaml an ambiguity arises: are paths
+inside a source file relative to the source file, current directory or
+pipeline.yaml?
 
+When we started working on Soopervisor, the concept of "project" arised,
+which is a folder that contains all necessary files to instantiate a
+Ploomber pipeline. To take current directory ambiguity out of the situation
+we decided that projects that use the Spec API should ignore the current
+directory and assume that *all relative paths in the spec are so to the
+pipeline.yaml parent folder* this means that we could initialize projects
+from anywhere by just pointing to the project's root folder (the one that
+contains pipeline.yaml). This is implemented in TaskSpec.py@init_product
+where we add the project's root folder to all relative paths. This implies
+that all relative paths are converted to absolute to make them agnostic
+from the current working directory. The basic idea is that a person
+writing a DAG spec should only have to know that paths are relative
+to the project's root folder and the DAGSpec should do the heavy lifting
+of instantiating a spec that could be converted to a dag (and reendered)
+independent of the working directory, because the project's root folder
+defines the pipelines's scope, and there is no point in looking for files
+outside of it, except when asking explicity through an absolute path.
+
+However, this behavior clashes with the Jupyter notebook defaults. Jupyter
+sets the current working directory not to the directory where
+"jupyter {notebook/lab}" was called but to the current file parent's.
+Our current logic would override this default behavior so we added an
+option to turn it back on (product_relative_to_source).
+
+Note that this does not affect the Python API, since that would create more
+confusion. The Python API should work as expected from any other library:
+paths should be relative to the current working directory.
+
+The only case where we can't determine a project's root is when a DAGSpec
+is initialized from a dictionary directly, but this is not something
+that end-users would do.
+
+Spec sections
+-------------
+"meta" is for settings that cannot be clearly mapped to the OOP Python
+interface, we don't call it config because there is a DAGConfig object in
+the Python API and this might cause confusion
 
 All other sections should represent valid DAG properties.
 """
@@ -43,65 +92,76 @@ class DAGSpec(MutableMapping):
     "location" key with the factory to call, the other explicitly describes
     the DAG structure as a dictionary.
 
+    When .to_dag() is called, the current working directory is temporarily
+    switched to the spec file parent folder (only applies when loading from
+    a file)
+
     Parameters
     ----------
-    data : dict
-        Specification to construct the DAG
+    data : str, pathlib.Path or dict
+        Path to a YAML spec or dict spec to construct the DAG
 
-    env : dict, pathlib.path or str
+    env : dict, pathlib.path or str, optional
         Dictionary or path/str to a YAML file with the environment to use,
         tags in any of the keys (i.e. {{some_tag}}) in "data" will be replaced
         by the corresponding values in "env". A regular :py:mod:`ploomber.Env`
-        object is created, see documentation for details
+        object is created, see documentation for details. If None and
+        data is a dict, no env is loaded. If None and loaded from a YAML spec,
+        an env.yaml file is loaded from the YAML spec parent folder, if it
+        exists
 
-    Notes
-    -----
-    Relative paths are heavily used for pointing to source files and products,
-    before the introduction of the Spec API, this was unambiguous: all paths
-    were relative to the current working directory, you only had to make sure
-    to start the session at the right place. Initially, the Spec API didn't
-    change a thing since the assumption was that one would convert a spec to
-    a DAG in a folder containing a pipeline.yaml file and paths in such file
-    would be relative to the file itself, no ambiguity. However, when we
-    introduced the Jupyter custom contents manager things changed because the
-    DAG can be initialized from directories where there isn't a pipeline.yaml
-    but we have to recursively look for one. Furthermore, if extract_upstream
-    or extract_product are True and the source files are not in the same folder
-    as pipeline.yaml an ambiguity arises: are paths inside a source file
-    relative to the source file or pipeline.yaml? To make this consistent,
-    all relative paths (whether in a pipeline.yaml or in a source file) are
-    relative to the pipeline.yaml folder but this can be changed with the
-    product_relative_to_source option. Furthermore, when using the custom
-    contents manager, we convert relative paths to absolute in special cases
-    to make sure they work. This is why we need to keep the pipeline.yaml
-    location when initializing a DAG from a spec.
+    lazy_import : bool, optional
+        Whether to import dotted paths to initialize PythonCallables with the
+        actual function. If False, PythonCallables are initialized directly
+        with the dotted path, which means some verifications such as import
+        statements in that function's module are delayed until the pipeline
+        is executed
     """
-    def __init__(self, data, env=None):
+    def __init__(self, data, env=None, lazy_import=False):
         if isinstance(data, (str, Path)):
-            self._parent_path = str(Path(data).parent)
+            path = data
+            # resolve the parent path to make sources and products unambiguous
+            # even if the current working directory changes
+            self._parent_path = str(Path(data).parent.resolve())
 
             with open(str(data)) as f:
                 data = yaml.load(f, Loader=yaml.SafeLoader)
+
+            env_path = Path(self._parent_path, 'env.yaml')
+
+            if env is None and env_path.exists():
+                env = str(env_path)
         else:
+            path = None
             self._parent_path = None
 
-        if isinstance(data, list):
-            data = {'tasks': data}
+        self.data = data
+
+        if isinstance(self.data, list):
+            self.data = data = {'tasks': self.data}
+
+        # validate keys defined at the top (nested keys are not validated here)
+        self._validate_top_keys(self.data, path)
 
         logger.debug('DAGSpec enviroment:\n%s', pp.pformat(env))
 
         if env is not None:
+            # NOTE: when loading from a path, EnvDict recursively looks
+            # at parent folders, this is useful when loading envs
+            # in nested directories where scripts/functions need the env
+            # but here, since we just need this for the spec, we might
+            # want to turn it off. should we add a parameter to EnvDict
+            # to control this?
             self.env = EnvDict(env)
-            data = expand_raw_dictionary(data, self.env)
+            self.data = expand_raw_dictionary(self.data, self.env)
         else:
             self.env = None
 
         logger.debug('Expanded DAGSpec:\n%s', pp.pformat(data))
 
-        load_from_factory = self._validate_top_level_keys(data)
-        self.data = data
-
-        if not load_from_factory:
+        # if there is a "location" top key, we don't have to do anything else
+        # as we will just load the dotted path when .to_dag() is called
+        if 'location' not in self.data:
             self.data['tasks'] = [
                 normalize_task(task) for task in self.data['tasks']
             ]
@@ -111,25 +171,32 @@ class DAGSpec(MutableMapping):
             # otherwise dynamic imports needed by TaskSpec will fail
             with add_to_sys_path(self._parent_path, chdir=False):
                 self.data['tasks'] = [
-                    TaskSpec(t, self.data['meta']) for t in self.data['tasks']
+                    TaskSpec(t,
+                             self.data['meta'],
+                             project_root=self._parent_path,
+                             lazy_import=lazy_import)
+                    for t in self.data['tasks']
                 ]
 
-    def _validate_top_level_keys(self, spec):
-        load_from_factory = False
+    def _validate_top_keys(self, spec, path):
+        """Validate keys at the top of the spec
+        """
+        if 'tasks' not in spec and 'location' not in spec:
+            path_ = f'(file: "{path}")' if self._parent_path else ''
+            raise KeyError('Invalid data to initialize DAGSpec, missing '
+                           f'key "tasks" {path_}')
 
         if 'location' in spec:
             if len(spec) > 1:
                 raise KeyError('If specifying dag through a "location" key '
                                'it must be the unique key in the spec')
-            else:
-                load_from_factory = True
         else:
             valid = {'meta', 'config', 'clients', 'tasks'}
             validate.keys(valid, spec.keys(), name='dag spec')
 
-        return load_from_factory
-
     def _validate_meta(self):
+        """Validate and instantiate the "meta" section
+        """
         if 'meta' not in self.data:
             self.data['meta'] = {}
 
@@ -168,7 +235,10 @@ class DAGSpec(MutableMapping):
         return dag
 
     def _to_dag(self):
-        # FIXME: validate that if there is location, there isn't anything else
+        """
+        Internal method to manage the different cases to convert to a DAG
+        object
+        """
         if 'location' in self:
             factory = load_dotted_path(self['location'])
             return factory()
@@ -329,7 +399,7 @@ def process_tasks(dag, tasks, dag_spec, root_path=None):
         if extract_prod:
             task_dict['product'] = source.extract_product()
 
-        task, up = task_dict.to_task(dag, root_path)
+        task, up = task_dict.to_task(dag)
 
         if extract_prod:
             logger.debug('Extracted product for task "%s": %s', task.name,
