@@ -312,7 +312,7 @@ class NotebookRunner(Task):
             raise ValueError('Can only use develop in notebooks loaded '
                              'from files, not from str')
 
-        nb = _read_rendered_notebook(self.source.rendered_nb_str)
+        nb = _read_rendered_notebook(self.source.nb_str_rendered)
 
         name = self.source.loc.name
         suffix = self.source.loc.suffix
@@ -322,7 +322,12 @@ class NotebookRunner(Task):
         tmp.write_text(content)
 
         # open notebook with injected debugging cell
-        _open_jupyter_notebook(str(tmp), app=app, args=args)
+        try:
+            subprocess.run(['jupyter', app, str(tmp)] +
+                           shlex.split(args or ''),
+                           check=True)
+        except KeyboardInterrupt:
+            print(f'Jupyter {app} application closed...')
 
         # read tmp file again, to see if the user made any changes
         content_new = Path(tmp).read_text()
@@ -366,7 +371,7 @@ class NotebookRunner(Task):
         if kind not in opts:
             raise ValueError('kind must be one of {}'.format(opts))
 
-        nb = _read_rendered_notebook(self.source.rendered_nb_str)
+        nb = _read_rendered_notebook(self.source.nb_str_rendered)
         _, tmp_path = tempfile.mkstemp(suffix='.py')
         code = jupytext.writes(nb, version=nbformat.NO_CONVERT, fmt='py')
         Path(tmp_path).write_text(code)
@@ -410,7 +415,7 @@ class NotebookRunner(Task):
 
         _, tmp = tempfile.mkstemp('.ipynb')
         tmp = Path(tmp)
-        tmp.write_text(self.source.rendered_nb_str)
+        tmp.write_text(self.source.nb_str_rendered)
 
         if self.local_execution:
             self.papermill_params['cwd'] = str(self.source.loc.parent)
@@ -458,24 +463,6 @@ chdir("{}")
     return nb
 
 
-def _open_jupyter_notebook(path, app, args):
-    """
-    Open notebook using the jupyter notebook application
-
-    Parameters
-    ----------
-    args : str
-        List of extra arguments, executed command becomes:
-        "jupyter notebook {path} {args}"
-    """
-    args = [f'"{token}"' for token in shlex.split(args or '')]
-
-    try:
-        subprocess.call(['jupyter', app, path] + args)
-    except KeyboardInterrupt:
-        print('Jupyter notebook server closed...')
-
-
 def _save():
     res = input('Notebook changed, do you want to save changes '
                 'in the original location? (injected parameters '
