@@ -60,26 +60,6 @@ def requires(pkgs, name=None, extra_msg=None):
     return decorator
 
 
-@requires(['matplotlib'])
-def path2fig(path_to_image, dpi=50):
-    # FIXME: having this import at the top causes trouble with the
-    # multiprocessing library, moving it here solves the problem but we
-    # have to find a better solution.
-    # more info: https://stackoverflow.com/q/16254191/709975
-    import matplotlib.pyplot as plt
-
-    data = plt.imread(path_to_image)
-    height, width, _ = data.shape
-    fig = plt.figure()
-    fig.set_size_inches((width / dpi, height / dpi))
-    ax = plt.Axes(fig, [0, 0, 1, 1])
-    ax.set_axis_off()
-    fig.add_axes(ax)
-    ax.imshow(data)
-
-    return fig
-
-
 def safe_remove(path):
     if path.exists():
         if path.is_file():
@@ -263,6 +243,17 @@ def load_dotted_path(dotted_path, raise_=True):
                 '[module_name].[function_name]'.format(dotted_path))
 
 
+def locate_dotted_path(dotted_path):
+    tokens = dotted_path.split('.')
+    module = '.'.join(tokens[:-1])
+    spec = importlib.util.find_spec(module)
+
+    if spec is None:
+        raise ModuleNotFoundError(f'Module {module!r} does not exist')
+
+    return spec
+
+
 def find_file_recursively(name, max_levels_up=6, starting_dir=None):
     """
     Find environment by looking into the current folder and parent folders,
